@@ -1,13 +1,12 @@
 // Scope Tracking
 
-use crate::language::InternSymbol;
-use crate::parser::lvalue::DeclType;
+use crate::language::{InternSymbol, Access};
 use crate::parser::stmt::Label;
 use crate::debug::symbol::DebugSymbol;
 use crate::codegen::JumpSite;
 use crate::codegen::opcodes::{LocalIndex, UpvalueIndex};
 use crate::codegen::funproto::UpvalueTarget;
-use crate::codegen::errors::{CompileResult, ErrorKind};
+use crate::codegen::errors::CompileResult;
 
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
@@ -18,19 +17,21 @@ pub(super) enum LocalName {
     // special local variables
     Receiver,  // inside a function call, this refers to the object that was called
     NArgs,     // inside a function call, the number of arguments passed at the call site
+    
+    Temp,      // anonymous temporaries
 }
 
 
 #[derive(Debug, Clone)]
 pub(super) struct Local {
-    decl: DeclType,
+    mode: Access,
     name: LocalName,
     index: LocalIndex,
     captured: bool, // tracks whether the local is being referenced by an upvalue
 }
 
 impl Local {
-    pub(super) fn decl(&self) -> DeclType { self.decl }
+    pub(super) fn mode(&self) -> Access { self.mode }
     pub(super) fn name(&self) -> LocalName { self.name }
     pub(super) fn index(&self) -> LocalIndex { self.index }
     pub(super) fn captured(&self) -> bool { self.captured }
@@ -162,18 +163,21 @@ impl Scope {
     }
     
     fn find_local_mut(&mut self, name: &LocalName) -> Option<&mut Local> {
+        if matches!(name, LocalName::Temp) {
+            return None; // temporaries should not be referenced
+        }
         self.locals.iter_mut().find(|local| local.name == *name)
     }
     
-    fn push_local(&mut self, decl: DeclType, name: LocalName) -> CompileResult<&Local> {
+    fn push_local(&mut self, mode: Access, name: LocalName) -> CompileResult<&Local> {
         let index = self.last_index().map_or(
             Ok(0),
             |index| index.checked_add(1)
-                .ok_or(ErrorKind::InternalLimit("local variable limit reached"))
+                .ok_or("local variable limit reached")
         )?;
         
         let local = Local {
-            decl, name, index, 
+            mode, name, index, 
             captured: false,
         };
         
@@ -181,13 +185,13 @@ impl Scope {
         Ok(self.locals.last().unwrap())
     }
     
-    fn insert_local(&mut self, decl: DeclType, name: LocalName) -> CompileResult<InsertLocal> {
+    fn insert_local(&mut self, mode: Access, name: LocalName) -> CompileResult<InsertLocal> {
         // see if this local already exists in the current scope
         if let Some(mut local) = self.find_local_mut(&name) {
-            (*local).decl = decl; // redeclare with new mutability
+            (*local).mode = mode; // redeclare with new mutability
             Ok(InsertLocal::HideExisting(local.index))
         } else {
-            self.push_local(decl, name)?;
+            self.push_local(mode, name)?;
             Ok(InsertLocal::CreateNew)
         }
     }
@@ -248,14 +252,14 @@ impl NestedScopes {
 
 #[derive(Debug, Clone)]
 pub(super) struct Upvalue {
-    decl: DeclType,
+    mode: Access,
     name: LocalName,
     index: UpvalueIndex,
     target: UpvalueTarget,
 }
 
 impl Upvalue {
-    pub(super) fn decl(&self) -> DeclType { self.decl }
+    pub(super) fn mode(&self) -> Access { self.mode }
     pub(super) fn name(&self) -> LocalName { self.name }
     pub(super) fn index(&self) -> UpvalueIndex { self.index }
     pub(super) fn target(&self) -> UpvalueTarget { self.target }
@@ -295,11 +299,11 @@ impl CallFrame {
     
     fn create_upval_for_local(&mut self, local: &mut Local) -> CompileResult<&Upvalue> {
         let index = UpvalueIndex::try_from(self.upvalues.len())
-            .map_err(|_| ErrorKind::InternalLimit("upvalue limit reached"))?;
+            .map_err(|_| "upvalue limit reached")?;
         
         let upval = Upvalue {
             index,
-            decl: local.decl,
+            mode: local.mode,
             name: local.name,
             target: UpvalueTarget::Local(local.index),
         };
@@ -312,11 +316,11 @@ impl CallFrame {
     
     fn create_upval_for_upval(&mut self, upval: &Upvalue) -> CompileResult<&Upvalue> {
         let index = UpvalueIndex::try_from(self.upvalues.len())
-            .map_err(|_| ErrorKind::InternalLimit("upvalue limit reached"))?;
+            .map_err(|_| "upvalue limit reached")?;
         
         let upval = Upvalue {
             index,
-            decl: upval.decl,
+            mode: upval.mode,
             name: upval.name,
             target: UpvalueTarget::Upvalue(upval.index),
         };
@@ -390,9 +394,9 @@ impl ScopeTracker {
     
     // local variables
     
-    pub(super) fn insert_local(&mut self, decl: DeclType, name: LocalName) -> CompileResult<InsertLocal> {
+    pub(super) fn insert_local(&mut self, mode: Access, name: LocalName) -> CompileResult<InsertLocal> {
         let scope = self.local_scopes_mut().current_scope_mut().expect("insert local in global scope");
-        scope.insert_local(decl, name)
+        scope.insert_local(mode, name)
     }
     
     pub(super) fn resolve_local(&self, name: &LocalName) -> Option<&Local> {
