@@ -5,7 +5,13 @@ use crate::parser::operator::BinaryOp;
 use crate::parser::expr::{Expr, ExprMeta};
 
 
-// LValues
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
+pub enum AssignType {
+    AssignLocal,
+    AssignNonLocal,
+    DeclImmutable,
+    DeclMutable,
+}
 
 #[derive(Debug, Clone)]
 pub enum LValue {
@@ -13,9 +19,16 @@ pub enum LValue {
     Attribute(Box<AttributeTarget>), // receiver, attribute name
     Index(Box<IndexTarget>), // receiver, index expression
     Tuple(Box<[LValue]>),
-    // Unpack(Box<LValue>),
+    PackItem(Option<Box<LValue>>),
+    
+    Modifier {
+        modifier: AssignType,
+        lvalue: Box<LValue>,
+    },
 }
 
+// LValue Data
+
 #[derive(Debug, Clone)]
 pub struct AttributeTarget {
     pub receiver: Primary,
@@ -33,25 +46,9 @@ pub struct IndexTarget {
 #[derive(Debug, Clone)]
 pub struct Assignment {
     pub lhs: LValue,
+    pub assign: AssignType,
     pub op: Option<BinaryOp>, // e.g. for +=, -=, *=, ...
     pub rhs: Expr,
-    pub nonlocal: bool,
-}
-
-// Declarations
-
-// TODO rename
-#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
-pub enum DeclType {
-    Immutable,
-    Mutable,
-}
-
-#[derive(Debug, Clone)]
-pub struct Declaration {
-    pub decl: DeclType,
-    pub lhs: LValue,
-    pub init: Expr,
 }
 
 // Convert expressions to LValues...
@@ -70,7 +67,20 @@ impl TryFrom<Atom> for LValue {
     fn try_from(atom: Atom) -> Result<Self, Self::Error> {
         match atom {
             Atom::Identifier(name) => Ok(LValue::Identifier(name)),
-            Atom::Group(expr) => (*expr).try_into(),
+            
+            Atom::Group { modifier, inner } => {
+                let lvalue = (*inner).try_into()?;
+
+                if let Some(modifier) = modifier {
+                    Ok(Self::Modifier {
+                        modifier,
+                        lvalue: Box::new(lvalue),
+                    })
+                } else {
+                    Ok(lvalue)
+                }
+            },
+
             _ => Err(IntoLValueError)
         }
     }
@@ -104,15 +114,20 @@ impl TryFrom<Expr> for LValue {
             
             Expr::Primary(primary) => primary.try_into(),
             
-            Expr::Tuple(expr_list) => {
-                let mut lvalue_list = Vec::<LValue>::new();
-                
-                for expr in expr_list.into_vec().into_iter() {
-                    let lvalue = expr.take_variant().try_into()?;
-                    lvalue_list.push(lvalue);
+            Expr::Ellipsis(Some(expr)) => {
+                let inner = LValue::try_from(*expr)?;
+                Ok(Self::PackItem(Some(Box::new(inner))))
+            }
+            Expr::Ellipsis(None) => Ok(Self::PackItem(None)),
+            
+            Expr::Tuple(items) if !items.is_empty() => {
+                let mut lvalue_items = Vec::new();
+                for expr in items.into_vec().into_iter() {
+                    let lvalue = LValue::try_from(expr.take_variant())?;
+                    lvalue_items.push(lvalue);
                 }
                 
-                Ok(Self::Tuple(lvalue_list.into_boxed_slice()))
+                Ok(Self::Tuple(lvalue_items.into_boxed_slice()))
             },
             
             _ => Err(IntoLValueError),
