@@ -1,5 +1,6 @@
 use crate::language::{IntType, FloatType, InternSymbol};
 use crate::parser::expr::{ExprMeta, Expr};
+use crate::parser::lvalue::AssignType;
 
 
 // Primary Expressions
@@ -10,12 +11,17 @@ pub enum Atom {
     EmptyTuple,
     // Self_,
     // Super,
+    
     Identifier(InternSymbol),
     BooleanLiteral(bool),
     IntegerLiteral(IntType),
     FloatLiteral(FloatType),
     StringLiteral(InternSymbol),
-    Group(Box<Expr>), // type annotation
+    
+    Group {
+        modifier: Option<AssignType>,
+        inner: Box<Expr>,
+    }
 }
 
 // These are the highest precedence operations in the language
@@ -25,10 +31,7 @@ pub enum AccessItem {
     
     Index(ExprMeta),
     
-    Invoke { 
-        args: Box<[ExprMeta]>, 
-        unpack: Option<ExprMeta> 
-    },
+    Invoke(Box<[ExprMeta]>),
     
     // Construct(ObjectConstructor),
 }
