@@ -1,16 +1,22 @@
+///! The Sphinx language garbage collector.
+///! Most of the "public" API is centered around the `Gc<T>` smart pointer. 
+///! See the documentation for the [runtime::gc::handle].
+
 use core::fmt;
 use core::ptr::NonNull;
 use core::cell::{Cell, RefCell};
 use log;
 
-mod data;
+mod trace;
+mod ptrmeta;
+mod gcbox;
 mod handle;
 mod weak;
 
-pub use data::GcTrace;
+pub use trace::GcTrace;
 pub use handle::{Gc, GcWeak};
 
-use data::{GcBox, GcBoxPtr};
+use gcbox::{GcBox, GcBoxPtr};
 
 
 thread_local! {
@@ -93,10 +99,10 @@ impl GcState {
     fn insert<T>(&mut self, mut gcbox: NonNull<GcBox<T>>) where T: GcTrace + ?Sized {
         unsafe {
             let size = gcbox.as_ref().header().size();
-            log::debug!("{:#X} allocate {} bytes", gcbox.as_ptr() as *const () as usize, size);
+            log::debug!("{:#X} allocate {} bytes for {:?}", gcbox.as_ptr() as *const () as usize, size, gcbox.as_ref().value());
             
             gcbox.as_mut().header_mut().set_next(self.boxes_start.take());
-            self.boxes_start = Some(GcBoxPtr::new(gcbox));
+            self.boxes_start = Some(gcbox.into());
             self.stats.allocated += size;
             self.stats.box_count += 1;
         }
@@ -140,7 +146,6 @@ impl GcState {
     unsafe fn sweep(&mut self) {
         let _guard = DropGuard::new();
         
-        //boxes_start: Option<NonNull<GcBox<dyn GcTrace>>>,
         let mut prev_box = None;
         let mut next_box = self.boxes_start;
         while let Some(mut gcbox) = next_box {
@@ -166,7 +171,7 @@ impl GcState {
 
 impl Drop for GcState {
     fn drop(&mut self) {
-        // unimplemented!()
+        unsafe { self.sweep() }
     }
 }
 
