@@ -3,7 +3,7 @@ use crate::language::{IntType, FloatType};
 use crate::runtime::Variant;
 use crate::runtime::function::Call;
 use crate::runtime::strings::{StringValue, static_symbol};
-use crate::runtime::errors::{ExecResult, ErrorKind};
+use crate::runtime::errors::{ExecResult, RuntimeError};
 
 
 mod ops;
@@ -13,11 +13,13 @@ mod boolean;
 mod numeric;
 mod string;
 mod tuple;
+mod iterator;
 mod misc;
 
 pub use tuple::Tuple;
-pub use misc::{Marker, NativeIterator, UserData};
+pub use misc::{Marker, UserData};
 pub use numeric::{int_from_str, float_from_str};
+pub use iterator::{IterState, UserIterator};
 
 use misc::Nil;
 
@@ -38,40 +40,44 @@ pub enum Type {
     Iterator,
     Metatable,
     Object,
+    Error,
     UserData,
 }
 
 impl Type {
-    pub fn name(&self) -> &'static str {
-        match self {
-            Self::Nil => "nil",
-            Self::Boolean => "bool",
-            Self::Marker => "marker",
-            Self::Integer => "int",
-            Self::Float => "float",
-            Self::String => "string",
-            Self::Tuple => "tuple",
-            Self::Function => "function",
-            Self::Iterator => "iterator",
-            Self::Metatable => "metatable",
-            Self::Object => "object",
-            Self::UserData => "userdata",
-        }
+    pub fn name(&self) -> StringValue {
+        let name = match self {
+            Self::Nil => static_symbol!("nil"),
+            Self::Boolean => static_symbol!("bool"),
+            Self::Marker => static_symbol!("marker"),
+            Self::Integer => static_symbol!("int"),
+            Self::Float => static_symbol!("float"),
+            Self::String => static_symbol!("string"),
+            Self::Tuple => static_symbol!("tuple"),
+            Self::Function => static_symbol!("function"),
+            Self::Iterator => static_symbol!("iterator"),
+            Self::Metatable => static_symbol!("metatable"),
+            Self::Object => static_symbol!("object"),
+            Self::Error => static_symbol!("error"),
+            Self::UserData => static_symbol!("userdata"),
+        };
+        name.into()
     }
 }
 
 impl fmt::Display for Type {
     fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
-        fmt.write_str(self.name())
+        self.name().with_str(|s| fmt.write_str(s))
     }
 }
 
+
 #[allow(unused_variables)]
 pub trait MetaObject {
     fn type_tag(&self) -> Type;
     
     fn type_name(&self) -> ExecResult<StringValue> {
-        Ok(StringValue::new_maybe_interned(self.type_tag().name()))
+        Ok(self.type_tag().name())
     }
     
     // formatting
@@ -84,11 +90,14 @@ pub trait MetaObject {
     fn as_float(&self) -> Option<ExecResult<FloatType>> { None }
     
     // iterators
-    fn next(&self) -> Option<ExecResult<Variant>> { None }
     
-    // data collections
+    // see iterator.rs for more detail on the iterator API
+    fn iter_init(&self) -> Option<ExecResult<IterState>> { None }
+    fn iter_next(&self, state: &Variant) -> Option<ExecResult<Variant>> { None }
+    fn iter_get(&self, state: &Variant) -> Option<ExecResult<Variant>> { None }
+    
+    // collections
     fn len(&self) -> Option<ExecResult<usize>> { None }
-    fn iter(&self) -> Option<ExecResult<Variant>> { None }
     //fn getitem(&self, item: &Variant) -> Option<ExecResult<Variant>> { None }
     //fn setitem(&self, item: &Variant) -> Option<ExecResult<Variant>> { None }
     
@@ -147,42 +156,48 @@ impl Variant {
     
     pub fn as_bits(&self) -> ExecResult<IntType> {
         self.as_meta().as_bits()
-            .ok_or_else(|| ErrorKind::MethodNotSupported(self.type_tag(), MethodTag::AsBits))?
+            .ok_or_else(|| RuntimeError::metamethod_not_supported(self, MethodTag::AsBits))?
     }
     
     pub fn as_int(&self) -> ExecResult<IntType> {
         self.as_meta().as_int()
-            .ok_or_else(|| ErrorKind::MethodNotSupported(self.type_tag(), MethodTag::AsInt))?
+            .ok_or_else(|| RuntimeError::metamethod_not_supported(self, MethodTag::AsInt))?
     }
     
     pub fn as_float(&self) -> ExecResult<FloatType> {
         self.as_meta().as_float()
-            .ok_or_else(|| ErrorKind::MethodNotSupported(self.type_tag(), MethodTag::AsFloat))?
+            .ok_or_else(|| RuntimeError::metamethod_not_supported(self, MethodTag::AsFloat))?
     }
 }
 
 impl Variant {
     pub fn len(&self) -> ExecResult<usize> {
         self.as_meta().len()
-            .ok_or_else(|| ErrorKind::MethodNotSupported(self.type_tag(), MethodTag::Len))?
+            .ok_or_else(|| RuntimeError::metamethod_not_supported(self, MethodTag::Len))?
     }
     
     pub fn is_empty(&self) -> ExecResult<bool> {
         Ok(self.len()? == 0)
     }
     
-    pub fn next(&self) -> ExecResult<Variant> {
-        self.as_meta().next()
-            .ok_or_else(|| ErrorKind::MethodNotSupported(self.type_tag(), MethodTag::Next))?
+    pub fn iter_init(&self) -> ExecResult<IterState> {
+        self.as_meta().iter_init()
+            .ok_or_else(|| RuntimeError::metamethod_not_supported(self, MethodTag::IterInit))?
     }
     
-    pub fn stop_iteration() -> Self {
-        Variant::marker(static_symbol!("StopIteration"))
+    pub fn iter_next(&self, state: &Variant) -> ExecResult<Variant> {
+        self.as_meta().iter_next(state)
+            .ok_or_else(|| RuntimeError::metamethod_not_supported(self, MethodTag::IterNext))?
+    }
+    
+    pub fn iter_get(&self, state: &Variant) -> ExecResult<Variant> {
+        self.as_meta().iter_get(state)
+            .ok_or_else(|| RuntimeError::metamethod_not_supported(self, MethodTag::IterItem))?
     }
     
     pub fn invoke(&self, args: &[Variant]) -> ExecResult<Call> {
         self.as_meta().invoke(args)
-            .ok_or_else(|| ErrorKind::MethodNotSupported(self.type_tag(), MethodTag::Invoke))?
+            .ok_or_else(|| RuntimeError::metamethod_not_supported(self, MethodTag::Invoke))?
     }
     
     pub fn fmt_echo(&self) -> ExecResult<StringValue> {
@@ -195,8 +210,9 @@ impl Variant {
 pub enum MethodTag {
     Invoke,
     Len,
-    Next,
-    Iter,
+    IterInit,
+    IterNext,
+    IterItem,
     AsBool,
     AsBits,
     AsInt,
@@ -210,8 +226,9 @@ impl MethodTag {
             Self::Invoke => "call",
             
             // iterators and iterables
-            Self::Next => "next",
-            Self::Iter => "iter",
+            Self::IterInit => "iter_init",
+            Self::IterNext => "iter_next",
+            Self::IterItem => "iter_get",
             
             // sequences
             Self::Len => "len",
