@@ -5,8 +5,8 @@ use crate::runtime::Variant;
 use crate::runtime::gc::Gc;
 use crate::runtime::function::{Call, Function, NativeFunction};
 use crate::runtime::strings::StringValue;
-use crate::runtime::types::{Type, MetaObject, Tuple, UserData, Nil, Marker, NativeIterator};
-use crate::runtime::errors::ExecResult;
+use crate::runtime::types::{Type, MetaObject, Tuple, UserData, Nil, Marker, IterState, UserIterator};
+use crate::runtime::errors::{ExecResult, RuntimeError};
 
 
 /// Newtype wrapper for `Variant` that impls `MetaObject` using enum-based static dispatch.
@@ -49,7 +49,9 @@ macro_rules! static_dispatch {
                 Variant::Function(fun) => <Gc<Function> as MetaObject>::$name(fun, $( $arg ),* ),
                 Variant::NativeFunction(fun) => <Gc<NativeFunction> as MetaObject>::$name(fun, $( $arg ),* ),
                 
-                Variant::Iterator(iter) => <Gc<dyn NativeIterator> as MetaObject>::$name(iter, $( $arg ),* ),
+                Variant::Error(error) => <Gc<RuntimeError> as MetaObject>::$name(error, $( $arg ),* ),
+                
+                Variant::Iterator(iter) => <Gc<dyn UserIterator> as MetaObject>::$name(iter, $( $arg ),* ),
                 
                 Variant::UserData(data) => <(dyn UserData + 'static) as MetaObject>::$name(&**data, $( $arg ),* ),
             }
@@ -70,11 +72,12 @@ impl MetaObject for MetaDispatch<'_> {
     static_dispatch!{ fn as_float() -> Option<ExecResult<FloatType>> }
     
     // iterators
-    static_dispatch!{ fn next() -> Option<ExecResult<Variant>> }
+    static_dispatch!{ fn iter_init() -> Option<ExecResult<IterState>> }
+    static_dispatch!{ fn iter_get(state: &Variant) -> Option<ExecResult<Variant>> }
+    static_dispatch!{ fn iter_next(state: &Variant) -> Option<ExecResult<Variant>> }
     
-    // data collections
+    // collections
     static_dispatch!{ fn len() -> Option<ExecResult<usize>> }
-    static_dispatch!{ fn iter() -> Option<ExecResult<Variant>> }
     
     // callable
     static_dispatch!{ fn invoke(args: &[Variant]) -> Option<ExecResult<Call>> }
