@@ -2,11 +2,11 @@ use core::fmt;
 use core::hash::{Hash, Hasher};
 use static_assertions::const_assert_eq;
 use crate::language::{IntType, FloatType};
-use crate::runtime::types::{Tuple, UserData, NativeIterator, Marker};
+use crate::runtime::types::{Tuple, UserData, UserIterator, Marker};
 use crate::runtime::function::{Function, NativeFunction};
 use crate::runtime::strings::{StringValue, StringSymbol, InlineStr};
 use crate::runtime::gc::{Gc, GcTrace};
-use crate::runtime::errors::{ExecResult, RuntimeError, ErrorKind};
+use crate::runtime::errors::{ExecResult, RuntimeError};
 
 
 // try to use less memory on 32-bit architectures
@@ -37,7 +37,9 @@ pub enum Variant {
     Function(Gc<Function>),
     NativeFunction(Gc<NativeFunction>),
     
-    Iterator(Gc<dyn NativeIterator>),
+    Iterator(Gc<dyn UserIterator>),
+    
+    Error(Gc<RuntimeError>),
     
     UserData(Gc<dyn UserData>),
 }
@@ -182,7 +184,7 @@ impl Variant {
             Self::InternStr(..) | Self::InlineStr(..) | Self::GCStr(..) =>
                 self.as_strval().unwrap().hash(state),
             
-            _ => return Err(ErrorKind::UnhashableValue(*self).into()),
+            _ => return Err(RuntimeError::unhashable_value(self)),
         }
         Ok(())
     }
@@ -198,7 +200,7 @@ impl<'a> TryFrom<&'a Variant> for VariantKey<'a> {
     type Error = Box<RuntimeError>;
     fn try_from(value: &'a Variant) -> ExecResult<Self> {
         if !value.can_hash() {
-            return Err(ErrorKind::UnhashableValue(*value).into());
+            return Err(RuntimeError::unhashable_value(value));
         }
         Ok(Self(value))
     }
